/* ============================================================================================================================================= /
 * LIBRARY:         Gestione funzioni grafiche per GLCD monocromatici
 * FILE:            GLCD_common_graphics.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         26/09/2017
 * UPDATED:         
 *
 * DESCRIP:         Funzioni grafiche per librerie GLCD monocromatici
 *                  Librerie supportate:
 *                  GLCD_KS0108B.c
 *                  OLED_SSD1306.c
 *
 * FUNCTIONS:       PRIVATE FUNCTION
 *                  GLCD_Plot8(unsigned char _x, unsigned char _y, unsigned char _dx, unsigned char _dy, unsigned char _corner, unsigned char _on_off);
 *
 *                  PUBLIC FUNCTIONS
 *                  GLCD_draw_line_x(unsigned char _x_start, unsigned char _x_dest, unsigned char _y, unsigned char _on_off);
 *                  GLCD_draw_y(unsigned char _y_origin, unsigned char _y_dest, unsigned char _x, unsigned char _on_off);
 *                  GLCD_draw_line_p2p (unsigned char _x_start, unsigned char _y_start, unsigned char _x_end, unsigned char _y_end, unsigned char _on_off);
 *                  GLCD_draw_line_angle(unsigned char _x_start, unsigned char _y_start, unsigned char _length,  unsigned int _angle, unsigned char _on_off);
 *                  GLCD_draw_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off);
 *                  GLCD_fill_rect (unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off);
 *                  GLCD_draw_circle(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _on_off);
 *                  GLCD_draw_arc(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _corner, unsigned char _on_off);
 *                  GLCD_fill_circle(unsigned char _x_centr,unsigned char _y_centr ,unsigned char _r, unsigned char _on_off);
 *                  GLCD_draw_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off);
 *                  GLCD_fill_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off);
 *                  GLCD_set_font_type (const unsigned char *_font_pointer);
 *                  GLCD_write_char(unsigned char _x, unsigned char _y, unsigned char _character, unsigned char _savespace, unsigned char _preserve_old, unsigned char _on_off);
 *                  GLCD_write_string (unsigned char _x, unsigned char _y, unsigned char _max_x, const unsigned char _string[], unsigned char _savespace, unsigned char _on_off);
 *                  GLCD_write_unsint(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned int _unsint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char void GLCD_write_sint(unsigned char _x, unsigned char _y, signed int _sint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned char _on_off);
 *                  GLCD_write_PBCD(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _pbcd, unsigned char _savespace, unsigned char _on_off);
 *                  GLCD_write_HEX(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _max_x, unsigned char _hex, unsigned char _savespace, unsigned int _on_off);
 *                  void GLCD_write_bit_string (unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _max_x, unsigned char _byte, unsigned char _savespace, unsigned int _on_off);
 *                  void GLCD_write_float (unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _max_x, float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _on_off);
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               26/09/2017          - Prima build completa
//! [History]
/* ============================================================================================================================================ */

#ifndef GLCD_COMMON_GRAPHICS
#define GLCD_COMMON_GRAPHICS

/* INCLUDES =================================================================================================================================== */
#include    "GLCD_common_graphics.h"
#include     <math.h>

/* Variabili globali libreria ================================================================================================================= */
    unsigned char font_width = 0;                                                       // larghezza del font 
    unsigned char font_height = 0;                                                      // altezza del font
    unsigned char font_space = 0;                                                       // spazio tra le lettere di una parola
    unsigned char font_first_char_offset = 0;                                           // offset per allineare il carattere char alla tabella in base al contenuto del font
    unsigned char font_vertical_byte = 0;                                               // numero di byte per altezza del carattere
    const unsigned char (*font_pointer) = 0;                                            // puntatore all'array
    unsigned int font_offset_btw_char = 0;                                              // offset tra i caratteri (serve per font con pi byte per l'altezza)                           
    
/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_line_x
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _x_end: coordinata x finale: 1~128 (unsigned char)
 *              _y: coordinata y: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una linea orizzontale su GLCD da punto _x_start a punto _x_end compresi su riga _y
/* ============================================================================================================================================ */
void GLCD_draw_line_x(unsigned char _x_start, unsigned char _x_end, unsigned char _y, unsigned char _on_off) {
    do {
        GLCD_draw_pixel ((_x_start), _y, _on_off);                                      // scrivo 1 bit a _x + lo spostamento a destra, y
        _x_start++;
    } while ((_x_start)<=_x_end);                                                       // fino a quando raggiungo x destinazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_line_y
 * Input:       _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _y_end: coordinata y finale: 1~64 (unsigned char)
 *              _x: coordinata x: 1~128 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una linea verticale su GLCD da punto _y_start a punto _y_end compresi su colonna _x
/* ============================================================================================================================================ */
void GLCD_draw_line_y(unsigned char _y_start, unsigned char _y_end, unsigned char _x, unsigned char _on_off) {
    do {
        GLCD_draw_pixel (_x, (_y_start), _on_off);                                      // scrivo 1 bit a _y + lo spostamento in basso, x
        _y_start++;
    } while ((_y_start)<=_y_end);                                                       // fino a quando raggiungo x destinazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_line_p2p
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _x_end: coordinata x finale: 1~128 (unsigned char)
 *              _y_end: coordinata y finale: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una linea da punto a punto su GLCD da punto _x/_y_start a punto x/_y_end compresi
/* ============================================================================================================================================ */
void GLCD_draw_line_p2p (unsigned char _x_start, unsigned char _y_start, unsigned char _x_end, unsigned char _y_end, unsigned char _on_off){
    unsigned char _x = _x_start;
    unsigned char _y = _y_start;
    signed char _x_inc1 = 0;
    signed char _x_inc2 = 0;
    signed char _y_inc1 = 0;
    signed char _y_inc2 = 0;
    unsigned char _den = 0;
    unsigned char _num = 0;
    unsigned char _numadd = 0;
    unsigned char _numpixels = 0;
    unsigned char _curpixel = 0;
    unsigned char _delta_x = 0;
    unsigned char _delta_y = 0;

    if (_y_end == _y_start) {                                                           // Se i 2 punti y sono uguali  devo disegnare una linea orizzontale
        if (_x_end > _x_start){                                                         // controllo la direzione
            GLCD_draw_line_x(_x_start, _x_end, _y_start, _on_off);
        } else {
            GLCD_draw_line_x(_x_end, _x_start, _y_start, _on_off);
        }
    } else if (_x_start == _x_end) {                                                    // Se i 2 punti x sono uguali devo disegnare una linea verticale
        if (_y_end > _y_start) {                                                        // controllo la direzione
            GLCD_draw_line_y(_y_start, _y_end, _x_start, _on_off);
        } else {
            GLCD_draw_line_y(_y_end, _y_start, _x_start, _on_off);
        }
    } else {                                                                            // altrimenti linea da punto a punto (routine trovata su internet e adattata)
        if (_x_end > _x_start) {                                                        // calcolo delta ed incrementi
            _delta_x = _x_end - _x_start +1;
            _x_inc1 = 1;
            _x_inc2 = 1;
        } else {
            _delta_x = _x_start - _x_end +1;
            _x_inc1 = -1;
            _x_inc2 = -1;
        }

        if (_y_end > _y_start) {
            _delta_y = _y_end - _y_start +1;
            _y_inc1 = 1;
            _y_inc2 = 1;
        } else {
            _delta_y = _y_start - _y_end +1;
            _y_inc1 = -1;
            _y_inc2 = -1;
        }

        if (_delta_x >= _delta_y) {
            _x_inc1 = 0;                                                                // Don't change the x when numerator >= denominator
            _y_inc2 = 0;                                                                // Don't change the y for every iteration
            _den = _delta_x;
            _num = _delta_x / 2;
            _numadd = _delta_y;
            _numpixels = _delta_x;                                                      // There are more x-values than y-values
        } else {                                                                        // There is at least one y-value for every x-value
            _x_inc2 = 0;                                                                // Don't change the x for every iteration
            _y_inc1 = 0;                                                                // Don't change the y when numerator >= denominator
            _den = _delta_y;
            _num = _delta_y / 2;
            _numadd = _delta_x;
            _numpixels = _delta_y;                                                      // There are more y-values than x-values
        }

        for (_curpixel = 0; _curpixel <= _numpixels; _curpixel++)
        {
            GLCD_draw_pixel (_x, _y, _on_off);
            _num = _num + _numadd;                                                      // Increase the numerator by the top of the fraction
            if (_num >= _den)                                                           // Check if numerator >= denominator
            {
              _num -= _den;                                                             // Calculate the new numerator value
              _x += _x_inc1;                                                            // Change the x as appropriate
              _y += _y_inc1;                                                            // Change the y as appropriate
            }
            _x += _x_inc2;                                                              // Change the x as appropriate
            _y += _y_inc2;                                                              // Change the y as appropriate
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_line_angle
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1-64 (unsigned char)
 *              _length: lunghezza: 1-128 (unsigned char)
 *              _angle: angolo: 0-360 (unsigned int)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una linea da punto origine di lunghezza stabilita con angolo su GLCD
/* ============================================================================================================================================ */
void GLCD_draw_line_angle(unsigned char _x_start, unsigned char _y_start, unsigned char _length,  unsigned int _angle, unsigned char _on_off) {
    GLCD_draw_line_p2p(_x_start, _y_start, (signed int)(_x_start + _length * cos(_angle * 0.017453292519)), (signed int)(_y_start + _length * sin(_angle * 0.017453292519)), _on_off);
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_rect
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _l: lunghezza: 1~128 (unsigned char)
 *              _h: altezza: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una rettangolo su GLCD
/* ============================================================================================================================================ */
void GLCD_draw_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off){
    unsigned char _data = 0;
    unsigned char _x_temp = 0;
    
    _l = _x_start + _l -1;                                                              // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h -1;
        
    if ((_y_start -1) >> 3 == (_y_start + _l) >> 3){                                    // righe orizzontali nella stessa pagina
        _data |= (_y_start - 1) & 0b00000111;
        _data |= _h & 0b00000111;
        _x_temp = _x_start - 1;
        
        do {
            GLCD_write_byte((_y_start -1) >> 3, _x_temp, _data, PRESERVE, _on_off);    
            _x_temp++;
        } while  ((_x_temp)<=_l);
          
    } else {
        GLCD_draw_line_x (_x_start, _l, _y_start, _on_off);
        GLCD_draw_line_x (_x_start, _l, _h, _on_off);
    }
   
    GLCD_draw_line_y (_y_start, _h, _x_start, _on_off);
    GLCD_draw_line_y (_y_start, _h, _l, _on_off);
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_fill_rect
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _l: lunghezza: 1~128 (unsigned char)
 *              _h: altezza: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna un rettangolo pieno su GLCD compreso tra i punti x y indicati (punti compresi)
/* ============================================================================================================================================ */
void GLCD_fill_rect (unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off) {
   
    _l = _x_start + _l -1;                                                              // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h -1;

    #ifdef GLCD_KS0108B    
    do {
        GLCD_draw_line_x (_x_start, _l, (_y_start), _on_off);
        _y_start++;
    } while ((_y_start)<= _h);                                                          // fino a quando raggiungo y destinazione
    #endif

    #ifdef OLED_SSD1306
    do {
        GLCD_draw_line_y (_y_start, _h, (_x_start), _on_off);
        _x_start++;
    } while ((_x_start) <= _l);
    #endif
}


/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_circle
 * Input:       _x_centr: coordinata x centro: 1~128 (unsigned char)
 *              _y_centr: coordinata y centro: 1-64 (unsigned char)
 *              _r: raggio 1-128 (unsigned char)
 *              _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 *
 * Overview:    Disegna una cerchio di raggio _r
 * Note:        Non implementata per OLED SPI e SSD1306 anche I2C
/* ============================================================================================================================================ */
#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
void GLCD_draw_circle(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _on_off){                 // routine trovata su internet e adattata

    unsigned char _x = 0;
    unsigned char _y = _r;
    signed int _p = 0;

    _p= -(_r/2);
    GLCD_Plot8(_x_centr, _y_centr, _x, _y, 0x0F, _on_off);
    while(_x<_y){
        _x++;
        if(_p<0)
            _p=_p+2*_x+1;
        else{
            _y--;
            _p=_p+2*(_x-_y)+1;
        }
        GLCD_Plot8(_x_centr,_y_centr, _x, _y, 0x0F, _on_off);
    }
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_arc
 * Input:       _x_centr: coordinata x del centro del cerchio: 1~128 (unsigned char)
 *              _y_centr: coordinata y del centro del cerchio: 1-64 (unsigned char)
 *              _r: raggio (unsigned char)
 *              _corner: angolo da disegnare
 *              _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 *
 * Overview:    Disegna un arco dato centro, raggio e angolo su display
 * Note:        Non implementata per OLED SPI e SSD1306 anche I2C
/* ============================================================================================================================================ */
#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
void GLCD_draw_arc(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _corner, unsigned char _on_off){                 // routine trovata su internet e adattata

    unsigned int _x = 0;
    unsigned int _y = _r;
    signed int _p = 0;

    _p= -(_r/2);
    GLCD_Plot8(_x_centr, _y_centr, _x, _y, _corner, _on_off);
    while(_x<_y){
        _x++;
        if(_p<0)
            _p=_p+2*_x+1;
        else{
            _y--;
            _p=_p+2*(_x-_y)+1;
        }
        GLCD_Plot8(_x_centr,_y_centr, _x, _y, _corner, _on_off);
    }
  
}

void GLCD_Plot8(unsigned char _x, unsigned char _y, unsigned char _dx, unsigned char _dy, unsigned char _corner, unsigned char _on_off){

    if (_corner & 0b0001){
        GLCD_draw_pixel (_x+_dx, _y-_dy, _on_off);    // 1 
        GLCD_draw_pixel (_x+_dy, _y-_dx, _on_off);    //
    }

    if (_corner & 0b0010){
        GLCD_draw_pixel (_x+_dx, _y+_dy, _on_off);    // 2
        GLCD_draw_pixel (_x+_dy, _y+_dx, _on_off);    //
    }
    
    if (_corner & 0b0100){
        GLCD_draw_pixel (_x-_dx, _y+_dy, _on_off);    // 3 
        GLCD_draw_pixel (_x-_dy, _y+_dx, _on_off);    //    
    }
    
    if (_corner & 0b1000){
        GLCD_draw_pixel (_x-_dx, _y-_dy, _on_off);    // 4
        GLCD_draw_pixel (_x-_dy, _y-_dx, _on_off);    //
    }
}     
#endif 

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_fill_circle
 * Input:       _x_centr: coordinata x centro: 1~128 (unsigned char)
 *              _y_centr: coordinata y centro: 1-64 (unsigned char)
 *              _r: raggio 1-128 (unsigned char)
 *              _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 *
 * Overview:    Disegna un cerchio pieno dato centro e raggio su display
 * Note:        Non implementata per OLED SPI e SSD1306 anche I2C
/* ============================================================================================================================================ */
#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
void GLCD_fill_circle(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _on_off){
    signed char _xoff =0;
    signed char _yoff = _r;
    signed char _balance = -_r;
    unsigned char _p0, _p1, _w0, _w1;

    while (_xoff <= _yoff) {
         _p0 = _x_centr - _xoff;
         _p1 = _x_centr - _yoff;
         _w0 = _xoff + _xoff;
         _w1 = _yoff + _yoff;

         GLCD_draw_line_x(_p0,_p0+_w0,_y_centr+_yoff,_on_off);
         GLCD_draw_line_x(_p0,_p0+_w0,_y_centr-_yoff,_on_off);
         GLCD_draw_line_x(_p1,_p1+_w1,_y_centr+_xoff,_on_off);
         GLCD_draw_line_x(_p1,_p1+_w1,_y_centr-_xoff,_on_off);

        if ((_balance += _xoff++ + _xoff)>= 0) {
            _balance  -= --_yoff+_yoff;
        }
    }
}    
#endif       

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_rounded_rect
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _l: lunghezza: 1~128 (unsigned char)
 *              _h: altezza: 1~64 (unsigned char)
 *              _r: raggio degli angoli: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una rettangolo con angoli arrotondati su display
 * Note:        Non implementata per OLED SPI e SSD1306 anche I2C
/* ============================================================================================================================================ */
#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
void GLCD_draw_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off){

    _l = _x_start + _l -1 - _r;                                                         // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h -1 - _r;
    
    GLCD_draw_line_x (_x_start +_r, _l, _y_start, _on_off);
    GLCD_draw_line_x (_x_start +_r, _l, _h+_r, _on_off);
    GLCD_draw_line_y (_y_start +_r, _h, _x_start, _on_off);
    GLCD_draw_line_y (_y_start +_r, _h, _l+_r, _on_off);
    
    GLCD_draw_arc(_l, _y_start + _r, _r, 1, _on_off);
    GLCD_draw_arc(_l, _h, _r, 2, _on_off);
    GLCD_draw_arc(_x_start + _r, _h, _r, 4, _on_off);
    GLCD_draw_arc(_x_start + _r, _y_start + _r, _r, 8, _on_off);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_fill_rounded_rect
 * Input:       _x_start: coordinata x di partenza: 1~128 (unsigned char)
 *              _y_start: coordinata y di partenza: 1~64 (unsigned char)
 *              _l: lunghezza: 1~128 (unsigned char)
 *              _h: altezza: 1~64 (unsigned char)
 *              _r: raggio degli angoli: 1~64 (unsigned char)
 *              _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Disegna una rettangolo pieno con angoli arrotondati su display
 * Note:        Non implementata per OLED SPI e SSD1306 anche I2C
/* ============================================================================================================================================ */
#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
void GLCD_fill_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off){
    
    GLCD_fill_rect(_x_start, _y_start + _r, _l, _h-(2*_r), _on_off);
    GLCD_fill_rect(_x_start + _r, _y_start, _l-(2*_r), _r, _on_off);
    GLCD_fill_rect(_x_start + _r, _y_start + _h -_r, _l-(2*_r), _r, _on_off);
    
    GLCD_fill_circle(_x_start + _r, _y_start +_r, _r, _on_off);
    GLCD_fill_circle(_x_start + _r, _y_start +_h -_r-1, _r, _on_off);
    GLCD_fill_circle(_x_start + _l -_r-1, _y_start +_h -_r-1, _r, _on_off);
    GLCD_fill_circle(_x_start + _l -_r-1,  _y_start +_r, _r, _on_off);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_font_type
 * Input:       *_font_pointer: puntatore all'array del font
 *
 * Overview:    Imposta le variabili a seconda del font selezionato
/* ============================================================================================================================================ */
void GLCD_set_font (const unsigned char *_font_pointer) {
    font_width = _font_pointer[0];
    font_height = _font_pointer[1];
    font_space = _font_pointer[2];
    font_first_char_offset = _font_pointer[3];
    font_vertical_byte = _font_pointer[4];
        
    font_pointer = _font_pointer;
    font_offset_btw_char = font_width * font_vertical_byte;                             // calcolo di quanti byte devo spostarmi nella tabella per ogni carattere
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_char
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _unsigned character: carattere da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _preserve_old: PRESERVE/OVERWRITE (unsigned char) Preserva o sovrascrive i dati sulla stessa pagina
 *              _on_off: scrittura positiva o negativa (unsigned char)
 * Output:      valore di x effettivamente scritti (unsigned char)      
 *
 * Overview:    Scrive un singlo carattere su GLCD
 * Precond:     Impostare il font tramite la funzione TFT_set_font()
 *              - la scrittura su GLCD viene fatta per pagina e non per bit attraverso la funzione GLCD_write_byte per velocizzare la scrittura.
/* ============================================================================================================================================ */
unsigned char GLCD_write_char(unsigned char _x, unsigned char _y, unsigned char _character, unsigned char _savespace, unsigned char _preserve_old, unsigned char _on_off) {      //indicare posizione bit piu basso
    unsigned char _i = 0;
    unsigned char _effective_x = 0;
    unsigned char _page_start = 0;
    unsigned char _page_max = 0;
    unsigned char _offset = 0;
    unsigned char _data[10] = {0};
    unsigned char _byte[12] = {0};
    unsigned int _index;
    unsigned char _b = 0;
    unsigned char _page_cur = 0;
    
    _index = _character - font_first_char_offset;                                       // tolgo l'offset dei caratteri dell'array 
    _index *= font_offset_btw_char;                                                     // calcolo il byte da prendere in base al carattere e all'offset tra i caratteri
    
    _y--;                                                                               // adatto x e y per avere valori che partono da 0
    _x--;

    _page_max = (_y + font_height -1) >> 3;                                             // ricavo la pagina pi alta (piu in basso sul display)
    _page_start = _y >> 3;                                                              // ricavo la pagina pi bassa (piu in alto sul display)
    _offset = _y & 0b00000111;                                                          // ricavo l'offset da aggiungere al primo byte (primo bit del byte da scrivere)

    if (_character >= 33 &&  _character <= 151){                                        // se carattere valido
        for (_i=0; _i<font_width; _i++) {                                                   // ciclo per N colonne
            _b = 0;

            // formo array con byte altezza singola colonna
            do{
                _data[_b] = font_pointer[(_index) + (_i * font_vertical_byte) + _b + 5];
                _b++;
            } while (_b < font_vertical_byte);

            // se la colonna ha dei valori
            if (_data[0]|_data[1]|_data[2]|_data[3]|_data[4]|_data[5]|_data[6]|_data[7]|_data[8]|_data[9] != 0){    

                // se il numero di pagine da scrivere  uguale ai byte letti sottraggo 1 per riportare il conteggio dei byte da 0
                if ((_page_max - _page_start + 1) <= _b)
                    _b--;

                // preparo i byte da scrivere a seconda dell'offset
                while (_b > 0){
                    _byte[_b] = (_data[_b] << _offset) | (_data[_b-1] >> (8-_offset));
                    _b--;
                }

                // ultimo byte
                _byte[0] = _data[0] << _offset;

                // scrivo i byte su display
                for (_page_cur = _page_start; _page_cur <= _page_max; _page_cur++) {
                    GLCD_write_byte(_page_cur, _x + _effective_x, _byte[_b], _preserve_old, _on_off);  // scrivo il byte
                    _b++;
                }
                _effective_x++; 
            }else if (_savespace == SAVESPACE_OFF){                                     // save space OFF cancello vecchie scritte
                _effective_x++; 
                if (_preserve_old == OVERWRITE)                                         // incremento prima perch poi clear 1 col decrementa x
                    GLCD_clear_1_col(_x+_effective_x, _y, _on_off);
            }
        }
    } else {                                                                            // carattere non valido scrivo spazio vuoto grande come il font
        for (unsigned char _m = 0; _m < font_width; _m++){ 
            _effective_x++;
            if (_preserve_old == OVERWRITE) 
                GLCD_clear_1_col(_x+_effective_x, _y, _on_off);

        }
    }
    return (_effective_x);
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_space_btw_chars
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 * Output:      valore di x effettivamente scritti (unsigned char)              
 *
 * Overview:    Riempie lo spazio tra i caratteri con il colore sfondo e torna il valore di x effettivamente scritte.
 * Precond:     Impostare il font tramite la funzione TFT_set_font()
/* ============================================================================================================================================ */
unsigned char GLCD_write_space_btw_chars (unsigned char _x, unsigned char _y, unsigned char _on_off){
    unsigned char _effective_x = 0;
    for (unsigned char _m = 0; _m < font_space; _m++){
        GLCD_clear_1_col(_x+_effective_x, _y, _on_off);    
        _effective_x++;
    }
    return _effective_x;    
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_clear_1_col
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *
 * Overview:    Cancella una colonna x
/* ============================================================================================================================================ */
void GLCD_clear_1_col(unsigned char _x, unsigned char _y, unsigned char _on_off){
    unsigned char _page_start = 0;
    unsigned char _page_max = 0;
    
    _x--;
    _y--;
    
    _page_max = (_y + font_height -1) >> 3;                                             // ricavo la pagina pi alta (piu in basso sul display)
    _page_start = _y >> 3;                                                              // ricavo la pagina pi bassa (piu in alto sul display)
    
    for (unsigned char _page_cur = _page_start; _page_cur <= _page_max; _page_cur++) {
        GLCD_write_byte(_page_cur, _x, 0x00, OVERWRITE, _on_off);                       // scrivo il byte
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_string
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _string: puntatore ad array di caratteri da scrivere, byte finale \0
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *              
 * Overview:    Scrive una stringa di caratteri su GLCD
/* ============================================================================================================================================ */
void GLCD_write_string (unsigned char _x, unsigned char _y, unsigned char _max_x, const unsigned char _string[], unsigned char _savespace, unsigned char _on_off) {
    unsigned char _i = 0;
    static unsigned char _incremental_x_pos = 0;                                        // variabile statica per il conteggio delle x effettivamente scritte
    unsigned char _preserve_old = 0;
    
    if (_max_x == 0)
        _preserve_old = PRESERVE;
    else 
        _preserve_old = OVERWRITE;

    while (_string[_i]!='\0') {                                                         // per ogni carattere della stringa fino a carattere nullo
        if (_string[_i] == ' ') {                                                       // se carattere spazio
            for (unsigned char _m = 0; _m < 4 * font_space; _m++){                      // riempio tante colonne in relazione al font_space (3*font_space)
                if (_preserve_old == OVERWRITE)
                    GLCD_clear_1_col(_x+_incremental_x_pos, _y, _on_off);
                _incremental_x_pos++;
            }
        } else {                                                                        // altrimenti scrivo il carattere e aumento di conseguenza la posizione x
            _incremental_x_pos += GLCD_write_char((_x + _incremental_x_pos), _y, _string[_i], _savespace, _preserve_old, _on_off);
            if (_preserve_old == OVERWRITE)
                _incremental_x_pos += GLCD_write_space_btw_chars(_x + _incremental_x_pos, _y, _on_off);
            else
                _incremental_x_pos += font_space;
        }
    _i++;
    }

    if (_preserve_old == OVERWRITE){
        while (_x + _incremental_x_pos < _x + _max_x){
            GLCD_clear_1_col(_x+_incremental_x_pos, _y, _on_off);
            _incremental_x_pos++;
        }
    }
    
    _incremental_x_pos = 0;                                                             // a fine stringa resetto la posizione di x
    
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_unsint
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _unsint: valore numerico unsigned int
 *              _number_digits: 1~5, numero di cifre da scrivere
 *              _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *              
 * Overview:    Scrive un numero unsigned int su GLCD dopo averlo convertito in array stringa di caratteri ASCII
/* ============================================================================================================================================ */
void GLCD_write_unsint(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned int _unsint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned char _on_off) {
    unsigned char _i = 0;
    unsigned char _c = 0;
    unsigned int _denom = 0;
    unsigned char _converted[5] = {0,0,0,0,0};
    unsigned char _converted_format[6] = {0,0,0,0,0,0};
    unsigned char _first_non_zero = 0;

    for (_i=0; _i<4; _i++) {                                                            // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

         _converted[_i] = _unsint/_denom;                                               // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _unsint = _unsint - (_denom * _converted[_i]);                              // sottraggo il valore gi  scritto
        }
    }
    _converted[4]= _unsint;                                                             // riporto il resto nell'array

    // in array _converted ho il dato completo di 0 iniziali

    _i = 0;
    for (_c = 0; _c < _number_digits-1 ; _c++) {                                        // ciclo per numero di cifre -1 (l'ultima la scrivo sempre

        if (_converted[_c + (5 -_number_digits)] != 0) {
            _converted_format[_i] = _converted[_c + (5 -_number_digits)] + 0b00110000;
            _first_non_zero = 1;
        } else {
            if (_zero_cleaning == ZERO_CLEANING_ON && _first_non_zero == 0)
                _i--;
            else
                _converted_format[_i] = '0';
        }
        _i++;
    }

    _converted_format[_i] =  _converted[4] + 0b00110000;
    _converted_format[_i+1] = '\0';                                                     // aggiungo valore di fine stringa
    GLCD_write_string(_x, _y, _max_x, _converted_format, _savespace, _on_off);          // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_sint
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _sint: valore numerico signed int
 *              _number_digits: 1~5, numero di cifre da scrivere
 *              _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *              
 * Overview:    Scrive un numero signed int su GLCD dopo averlo convertito in array stringa di caratteri ASCII
/* ============================================================================================================================================ */
void GLCD_write_sint (unsigned char _x, unsigned char _y, unsigned char _max_x, signed int _sint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned char _on_off) {
    unsigned char _incremental_x_pos = 0;
    unsigned char _preserve_old = 0;
    
    if (_max_x == 0)
        _preserve_old = PRESERVE;
    else 
        _preserve_old = OVERWRITE;
    
    if (_sint <0) {                                                                     // se < 0
        _incremental_x_pos += GLCD_write_char(_x, _y, '-', _savespace, _preserve_old, _on_off);        // scrivo il segno -
        if (_preserve_old == OVERWRITE)
            _incremental_x_pos += GLCD_write_space_btw_chars(_x + _incremental_x_pos, _y, _on_off);
        else 
            _incremental_x_pos += font_space;
        GLCD_write_unsint((_x + _incremental_x_pos), _y, _max_x, ~(_sint-1), _number_digits, _zero_cleaning, _savespace, _on_off);  // scrivo sul display il complemento a 1 + 1
    } else {                                                                            // se positivo
        _incremental_x_pos += GLCD_write_char(_x, _y, '+', _savespace, _preserve_old, _on_off);        // scrivo il segno -
        if (_preserve_old == OVERWRITE)
            _incremental_x_pos += GLCD_write_space_btw_chars(_x + _incremental_x_pos, _y, _on_off);
        else
            _incremental_x_pos += font_space;    
        GLCD_write_unsint(_x + _incremental_x_pos, _y, _max_x, _sint, _number_digits, _zero_cleaning, _savespace, _on_off);                 // tratto come se fosse un unsigned int
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_PBCD
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _pbcd: valore numerico codificato in packed BCD
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *               
 * Overview:    Scrive un numero codificato in 1 byte packed BCD su GLCD (non salva spazio)
/* ============================================================================================================================================ */
void GLCD_write_PBCD(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _pbcd, unsigned char _savespace, unsigned char _on_off) {
    unsigned char _string[3];
    
    _string[0] = ((_pbcd & 0b11110000) >> 4) + '0';                                     // decine = 4 bit alti del dato in ingresso
    _string[1] = (_pbcd & 0b00001111) + '0';                                            // unit  = 4 bit bassi del byte in ingresso;
    _string[2] = '\0';
    
    GLCD_write_string(_x, _y, _max_x, _string, _savespace, _on_off);                    // scrivo la stringa dei valori
}


/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_HEX
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se GLCD_PORTRAIT, 1~320 se GLCD_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se GLCD_PORTRAIT, 1~240 se GLCD_LANDSCAPE (unsigned int)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _hex: valore da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _on_off: colore del carattere (unsigned int)
 *
 * Overview:    Scrive un byte HEX su display
/* ============================================================================================================================================ */
void GLCD_write_HEX(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _hex, unsigned char _savespace, unsigned int _on_off){
    unsigned char _H = 0;                                                               // decine
    unsigned char _L = 0;                                                               // unit 
    unsigned char _string[3];
    
    _L = _hex & 0b00001111;                                                             // unit  = 4 bit bassi del byte in ingresso
    _H = (_hex & 0b11110000) >> 4;                                                      // decine = 4 bit alti del dato in ingresso

    if (_H < 10)
        _string[0] = _H | 0b00110000;
    else
        _string[0] = _H + 0b00110111;
    
    if (_L < 10)
        _string[1] = _L | 0b00110000;
    else
        _string[1] = _L + 0b00110111;
    
    _string[2] = '\0';
    
    GLCD_write_string(_x, _y, _max_x, _string, _savespace, _on_off);                    // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_bit_string
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _byte: valore da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _on_off: colore del carattere (unsigned int)
 *
 * Overview:    Scrive un byte come stringa di bit 1-0 su display
/* ============================================================================================================================================ */
void GLCD_write_bit_string (unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _byte, unsigned char _savespace, unsigned int _on_off) {
    unsigned char _string[9] = {0};
    
    for (unsigned char _i = 0; _i<8; _i++){
        if (_byte & 0x80)
            _string[_i] = '1';
        else 
            _string[_i] = '0';
        
        _byte <<= 1;
    }
    
    _string[8] = '\0';
    
    GLCD_write_string(_x, _y, _max_x, _string, _savespace, _on_off);                    // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_float
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 *              _float: valore da scrivere (float)
 *              _digits_int: numero di cifre interi da scrivere (1~5, unsigned char) 
 *              _digits_part: numero di cifre decimali da scrivere (1~10, unsigned char)
 *              _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 *              _on_off: scrittura positiva o negativa (unsigned char)
 *
 * Overview:    Scrive un valore float su display
/* ============================================================================================================================================ */
void GLCD_write_float (unsigned char _x, unsigned char _y, unsigned char _max_x, float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _on_off) {
    signed int _int = 0;
    float _part = 0;
    unsigned char _converted[5] = {0};
    unsigned char _string_part[10] = {0};
    unsigned char _string_compl[18] = {0};                                              // stringa finale segno + max 5 interi + , + max 10 decimali + carattere di fine stringa
    unsigned char _string_index = 0;
    
    unsigned int _denom = 0;
    unsigned char _first_non_zero = 0;
    
    if (_digits_int > 5)                                                                // valore massimo di interi
        _digits_int = 5;
    
    if (_digits_part > 10)                                                              // valore massimo di decimali = 10
        _digits_part = 10;
        
    if (_float < 0){                                                                    // valore negativo
        _float *= -1;                                                                   // porto positivo
       _string_compl[0] = '-';                                                          // e scrivo segno meno nella stringa finale
       _string_index++;
    }
    
    _int = (int) _float;                                                                // estraggo la parte intera
    _part = _float - (float)_int;                                                       // lascio solo la parte decimale
    
    // converto decimali in stringa
    for (unsigned char _i = 0; _i < _digits_part +1; _i++ ){                            // per ogni decimale richiesto +1 per calcolo approssimazione
        _part = _part *10;                                                              // moltiplico per 10
        _string_part[_i] = (unsigned char)_part;                                        // estraggo la parte intera
        _part = _part - (float) _string_part[_i];                                       // dalla parte decimale sottraggo la nuova intera
                                                     
    }    
    
    // arrotondamento decimali
    if (_digits_part == 0) {                                                            // 0 cifre decimali
        if (_string_part[0] > 4){                                                       // se la prima decimale  maggiore di 4 aggiungo 1 agli interi
            if (_int >= 0)                                                              // se positivo
                _int++;
            else                                                                        // altrimenti sottraggo 1
                _int--;    
        }
    } else {                                                                            // se ho pi cifre decimali
        if (_string_part[_digits_part] > 4){                                            // se l'ultima decimale (aggiunta solo per l'arrotondamento)  maggiore di 4
                _string_part[_digits_part - 1]++;                                       // aggiungo uno alla decimale precedente
        }
        
        for (unsigned char _i = 0; _i < _digits_part -1; _i++ ){                        // controlla dalla penultima decimale alla prima
            if (_string_part[_digits_part - 1 - _i] > 9){                               // se maggiore di 9
                _string_part[_digits_part - 1 - _i] = 0;                                // la porto a 0
                _string_part[_digits_part - 2 - _i]++;                                  // e aggiungo uno alla precedente
            }
        }

        if (_string_part[0] > 9){                                                       // se la prima decimale  maggiore di 9
            _string_part[0] = 0;                                                        // la azzero e incremento o decremento gli interi
            if (_int >= 0)
                _int++;
            else
                _int--;
        }
    
    }
    
     // conversione intero in stringa
    for (unsigned char _i=0; _i<4; _i++) {                                              // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

        _converted[_i] = _int/_denom;                                                   // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _int = _int - (_denom * _converted[_i]);                                    // sottraggo il valore gi  scritto
        }
    }
    _converted[4]= _int;                                                                // riporto il resto nell'array

                                                                                        // in array _converted ho il dato completo di 0 iniziali
                                                                                        // sposto in stringa finale solo le cifre massime definite da _digits_int e da _zero_cleaning 
    for (unsigned char _c = 0; _c < _digits_int-1 ; _c++) {                             // ciclo per numero di cifre -1 (l'ultima la scrivo sempre)

        if (_converted[_c + (5 -_digits_int)] != 0) {
            _string_compl[_string_index] = _converted[_c + (5 -_digits_int)] + 0b00110000;
            _first_non_zero = 1;
        } else {
            if (_zero_cleaning == ZERO_CLEANING_ON && _first_non_zero == 0)
                _string_index--;
            else
                _string_compl[_string_index] = '0';
        }
        _string_index++;
    }
    _string_compl[_string_index] =  _converted[4] + 0b00110000;
    
    _string_index++;
    _string_compl[_string_index] = ',';                                                 // salvo carattere , nella stringa
    _string_index++;
    
    for (unsigned char _i = 0; _i < _digits_part; _i++){
        _string_compl[_string_index] = _string_part[_i] |= 0b00110000;                  // sposto i valori dall'array _string_part a _string_compl convertendolo in carattere ASCII
        _string_index++;
    }
    
    _string_compl[_string_index] = '\0';                                                // aggiungo carattere di fine stringa
    
    
    GLCD_write_string(_x , _y, _max_x, _string_compl, _savespace, _on_off);             // scrivo la stringa dei valori
}     

#endif