/* ============================================================================================================================================= /
 * LIBRARY:         Gestione PGA2311, PGA2310, PGA2320
 * FILE:            PGA23xx.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         06/11/2016
 * UPDATED:         
 *
 * DESCRIP:         Gestione PGA2311, PGA2310, PGA2320
 *
 * FUNCTIONS:       PGA23xx_enable()
 *                  PGA23xx_disable() 
 *                  PGA23xx_send_register_value()
 *                  PGA23xx_mute()
 *                  PGA23xx_zcen()
 *                  PGA23xx_calc_db()
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               11/08/2017          - Prima release
//! [History]
/* ============================================================================================================================================ */

#ifndef PGA23xx
#define PGA23xx

/* Variabili globali libreria ================================================================================================================= */

/* INCLUDES =================================================================================================================================== */
#include    "PGA23xx_settings.h"
#include    "PGA23xx.h"
/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_enable
 *
 * Overview:    Abilita il modulo SPI e ne imposta i parametri
/* ============================================================================================================================================ */
void PGA23xx_enable(void){   
    _PGA23xx_SPI_disable();
    _PGA23xx_SPI_enable(SAMPLE_AT_MIDDLE, TRA_IDLE_TO_ACTIVE, IDLE_IS_HIGH, PGA23xx_SPI_CLOCK);
    PGA23xx_CS = 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_disable
 *
 * Overview:    Disabilita il modulo SPI
/* ============================================================================================================================================ */    
void PGA23xx_disable(void){              
    _PGA23xx_SPI_disable();
    PGA23xx_CS = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_send_register_value
 * Input:       _r: valore da impostare nel registro per il canale sinistro (unsigned char 0 = mute, 1= -95.5db 255 = +31.5db)
 *              _l: valore da impostare nel registro per il canale destro (unsigned char 0 = mute, 1= -95.5db 255 = +31.5db)
 * Output:      -1: bus collision 
 *
 * Overview:    invia i valori al chip 
 * ============================================================================================================================================ */   
signed char PGA23xx_send_register_value(unsigned char _r, unsigned char _l){
    signed char _return;
    PGA23xx_CS = 0;
    _return = _PGA23xx_SPI_send_byte(_r);
    _return |= _PGA23xx_SPI_send_byte(_l);
    PGA23xx_CS = 1;
    
    return _return;
} 

/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_hw_mute
 * Input:       _on_off: abilitazione o disabilitazione MUTE: PGA23xx_MUTE_ON/PGA23xx_MUTE_OFF (unsigned char)
 *
 * Overview:    Abilita o disabilita mute via hardware pin
 * ============================================================================================================================================ */
#ifdef PGA23xx_MUTE
void PGA23xx_hw_mute(unsigned char _on_off){
    PGA23xx_MUTE = _on_off;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_zcen
 * Input:       _on_off: abilitazione o disabilitazione zero crossing: PGA23xx_ZCEN_ON/PGA23xx_ZCEN_OFF (unsigned char)
 *
 * Overview:    Abilita o disabilita zero crossing via hardware pin
 * ============================================================================================================================================ */
#ifdef PGA23xx_ZCEN
void PGA23xx_zcen(unsigned char _on_off){
    PGA23xx_ZCEN = _on_off;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    PGA23xx_calc_db
 * Input:       _register_value: Valore inviato al registro (per canale) (unsigned char)
 * Output:      valore in db dell'attenuazione/guadagno
 *
 * Overview:    Restituisce il valore in dB dal valore inviato al registro
 * ============================================================================================================================================ */
float PGA23xx_calc_db(unsigned char _register_value){
    return 31.5 - (0.5*(255-_register_value));
}

#endif