/* ============================================================================================================================================= /
 * LIBRARY:         Gestione OLED SSD1306/SH1106
 * FILE:            OLED_SSD1306.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         21/08/2017
 * UPDATED:         
 *
 * DEPENDENCE:      Richiede libreria Delay.c, libreria SPI_init.c o I2C_init.c
 *                  impostazioni in OLED_SSD1306_settings.h
 * DESCRIP:         Parametri e prototipi per libreria OLED_SSD1306.c
/* ============================================================================================================================================ */

/* COMANDI OLED =============================================================================================================================== */
// registri e comandi    
#define OLED_SET_CONTRAST               0x81
#define OLED_ALL_PIXELS_ON              0xA5
#define OLED_ALL_PIXELL_RESUME_RAM      0xA4
#define OLED_SET_NORMAL                 0xA6
#define OLED_SET_INVERSE                0xA7
#define OLED_ON                         0xAF
#define OLED_OFF                        0xAE    
#define OLED_SET_DISP_CLOCK             0xD5
#define OLED_SET_MUX_RATIO              0xA8 
#define OLED_SET_DISP_OFFSET            0xD3 
#define OLED_SET_START_LINE             0x40
#define OLED_CHARGE_PUMP_REG            0x8D
#define OLED_SEGMENT_REMAP_OFF          0xA0
#define OLED_SEGMENT_REMAP_ON           0xA1
#define OLED_COM_SCAN_DIR_NOR           0xC0
#define OLED_COM_SCAN_DIR_INV           0xC8
#define OLED_ADDRESSING_MODE_REG        0x20
#define OLED_COM_HW_CONFIG              0xDA 
#define OLED_SET_PRECH_PERIOD           0xD9 
#define OLED_SET_VCOM_DETECT            0xDB    
    
#define OLED_ADDRESSING_HORIZONTAL      0x00
#define OLED_ADDRESSING_VERTICAL        0x01
#define OLED_ADDRESSING_PAGE            0x02 

#define OLED_SCROLL_ENABLED             0x2F
#define OLED_SCROLL_DISABLED            0x2E

#define OLED_HORIZONTAL_SCROLL_RIGHT    0x26
#define OLED_HORIZONTAL_SCROLL_LEFT     0x27
#define OLED_VERTICAL_HORIZONTAL_RIGHT  0x29
#define OLED_VERTICAL_HORIZONTAL_LEFT   0x2A


#ifdef OLED_MODE_I2C
    #define OLED_I2C_ADDR               0x78                                            // indirizzo SSD1306
    #define OLED_I2C_COMMAND            0x00
    #define OLED_I2C_DATA               0x40
#endif

/* GENERAL CONST ============================================================================================================================== */
#define HIGH                0b00000001
#define LOW                 0b00000000
#define PIXEL_ON            0b00000001
#define PIXEL_OFF           0b00000000

#define ZERO_CLEANING_ON    1
#define ZERO_CLEANING_OFF   0
#define SAVESPACE_ON        1
#define SAVESPACE_OFF       0
#define PRESERVE            0
#define OVERWRITE           1

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifdef OLED_MODE_SPI_4
    #ifndef SPI
        #error  Includere SPI_init.c
    #endif
#elif defined OLED_MODE_I2C
    #ifndef I2C 
        #error  Includere I2C_init.c
    #endif
#else
    #error Definire il protocollo di comunicasione in OLED_SSD1306_settings.h
#endif

#if defined OLED_PORTRAIT || defined OLED_PORTRAIT_180
#else
    #error Definire l'orientamento del display
#endif

#if defined OLED_DRIVER_SSD1306 || defined OLED_DRIVER_SH1106
#else
    #error Definire il driver utilizzato
#endif

/* RIDEFINIZIONE NOME FUNZIONI A SECONDA DEL MODULO USATO ===================================================================================== */
#ifdef OLED_MODE_SPI_4
    #ifdef OLED_SPI_MODULE_1                                                            // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
        #ifndef SPI_MODULE_1
            #error SPI_MODULE_1 non abilitato in SPI_init_settings.h
        #endif

        #define _OLED_SPI_enable         SPI_enable
        #define _OLED_SPI_disable        SPI_disable
        #define _OLED_SPI_send_byte      SPI_send_byte
        
    #elif defined OLED_SPI_MODULE_2
        #ifndef SPI_MODULE_2
            #error SPI_MODULE_2 non abilitato in SPI_init_settings.h
        #endif

        #define _OLED_SPI_enable         SPI2_enable
        #define _OLED_SPI_disable        SPI2_disable
        #define _OLED_SPI_send_byte      SPI2_send_byte
        
    #else
        #error Definire il modulo SPI da utilizzare per SSD1306
    #endif
#endif

#ifdef OLED_MODE_I2C
    #ifdef OLED_I2C_MODULE_1                                                            // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
        #ifndef I2C_MODULE_1
            #error I2C_MODULE_1 non abilitato in I2C_init_settings.h
        #endif
        #define _OLED_I2C_1ByteAddr_1ByteData_write     I2C_1ByteAddr_1ByteData_write
        #define _OLED_I2C_1ByteAddr_1ByteData_read      I2C_1ByteAddr_1ByteData_read
        
    #elif defined OLED_I2C_MODULE_2
        #ifndef I2C_MODULE_2
            #error I2C_MODULE_2 non abilitato in I2C_init_settings.h
        #endif
        #define _OLED_I2C_1ByteAddr_1ByteData_write     I2C2_1ByteAddr_1ByteData_write
        #define _OLED_I2C_1ByteAddr_1ByteData_read      I2C2_1ByteAddr_1ByteData_read
        
    #else
        #error Definire il modulo I2C da utilizzare per SSD1306
    #endif
#endif

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file OLED_SSD1306.c
 * Libreria per la gestione di display grafici OLED con risoluzione 128x64, 128x32 pixel con controller SSD1306 o SH1106. Le differenze tra i 2 driver OLED
 * sono minime e gestite tramite controlli di compilazione in base al driver definito nelle impostazioni.
 * Entrambi i driver possono essere utilizzati sia con BUS SPI a 4 fili che con BUS I2C.
 * Nella selezione del driver e del BUS di comunicazione tenere presente le caratteristiche e limitazioni elencate sotto.
 * In questa libreria sono presenti solo le funzioni di base di gestione del display. Le funzioni di grafica avanzate e di scrittura caratteri, testi e numeri sono nella libreria
 * condivisa GLCD_common_graphics.c che viene automaticamente inclusa anche in questa libreria.
 * - Caratteristiche e limitazioni del Driver e del BUS di comunicazione:
 *  + SPI nettamente pi veloce che I2C
 *  + SSD1306 ha alcune funzioni di scrolling del display, totale o parziale
 *  + SH1106 con BUS I2C pu rileggere i dati dal display, sono abilitate alcune funzioni aggiuntive:
 *   - GLCD_draw_circle
 *   - GLCD_draw_arc
 *   - GLCD_fill_circle
 *   - GLCD_draw_rounded_rect
 *   - GLCD_fill_rounded_rect
 *   - Le funzioni di scrittura testi possono avere caratteri che ricadono parzialmente sulla stessa pagina senza che questi si sovrascrivano a vicenda.
 *  + SH1106 pu pilotare display da 132x64 ma solitamente  utilizzato con display da 128x64 con ram mappata da 2 a 130. Agire sulla costante SH1106_X_OFFSET
 *    per utilizzare driver impostati diversamente.
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c prima delle libreria OLED_SSD1306.c
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare il driver utilizzato (OLED_DRIVER_SSD1306 o OLED_DRIVER_SH1106) in OLED_SSD1306_settings.h
 *  + Impostare il modulo di comunicazione utilizzato (SPI/I2C) in OLED_SSD1306_settings.h
 *  + Abilitare il modulo SPI o I2C utilizzato in SPI_init_settings.h o I2C_init_settings.h
 *  + Impostare i parametri di funzionamento a seconda del protocollo utilizzato in OLED_SSD1306_settings.h 
 *  + Se si utilizzano pi device SPI disabilitare il modulo SPI tramite funzione GLCD_disable() e riabilitarlo per il nuovo modulo
 *  (i parametri di comunicazione potrebbero essere diversi). In alternativa impostare direttamente nel programma principale il modulo SPI con le funzioni
 *  SPI_enable() e SPI2_enable()
 *  + Se si utilizza il Bus I2C impostare la variabile oled_i2c_dev_addr a seconda dell'indirizzo del display (PIN SA0-D/C). E' possibile usare 2 display contemporaneamente impostando
 *    di volta in volta la variabile con l'indirizzo corretto.
 *  + Se si utilizza il protocollo I2C inizializzare il Bus tramite funzione I2C_init_master(400); e poi inizializzare il display con la funzione GLCD_init();
 *  + Informazioni approfondite sulla libreria e sulle funzioni grafiche si possono trovare nel file <a href="md__c_1__working_space__progetti__librerie__c__m_d_files__g_l_c_d_readme.html" target="_self"><b>GLCD_readme.mb</b></a>
 * - Files:
 *  + OLED_SSD1306.c: Codice funzioni
 *  + OLED_SSD1306.h: Parametri e prototipi
 *  + OLED_SSD1306_settings.h: Impostazioni specifiche per progetto
 */
 
/**
 * Invia i byte di comando al display
 * @param   _command: comando da inviare (unsigned char)
 * @return  void
 */
void GLCD_send_command (unsigned char _command);

/**
 * Invia i byte da scrivere in GRAM al display 
 * @param   _data: dato da scrivere su display (unsigned char)
 * @return  void
 */
void GLCD_send_data(unsigned char _data);

/**
 * Legge il dato scritto alla pagina/colonna precedentemente indirizzate
 * @param   void
 * @return  Dato letto dal display
 * @note    Solo per SH1106 in comunicazione I2C 
 */
#if defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C
unsigned char GLCD_read_data (void);
#endif

/**
 * Attiva o disattiva il display
 * @param   _status: ON o OFF (unsigned char) 
 * @return  void
 */
void GLCD_set_display(unsigned char _status);

/**
 * Inizializzazione display
 * @param   void
 * @return  void
 * @pre     Definire l'orientamento del display in GLCD_SSD1306_settings.h
 */
void GLCD_init(void);

/**
 * Disabilita il modulo SPI
 * @param   void
 * @return  void
 */
void GLCD_disable(void);

/**
 * Imposta il contrasto del display
 * @param   _contrast: valore del contrasto 0~255 (unsigned char)
 * @return  void
 */ 
void GLCD_set_contrast(unsigned char _contrast);

/**
 * Abilita o disabilita la scrittura in negativo 
 * @param   _on_off: ON: scritte nere sfondo bianco, OFF scritte bianche sfondo nero (unsigned char)
 * @return  void
 */
void GLCD_negative_write(unsigned char _on_off);

/**
 * Imposta il puntatore GRAM alla pagina e colonna desiderate
 * @param   _page: valore della pagina (0 ~ MAX_PAGES) (unsigned char)
 * @param   _column: valore della colonna (0 ~ X_RESOLUTION) (unsigned char)
 * @return  void
 */ 
void GLCD_set_pointer(unsigned char _page, unsigned char _column);

/**
 * Imposta la modalit di scrittura del display
 * @param   _mode: modalit di indirizzamento (unsigned char)
 *              - OLED_ADDRESSING_HORIZONTAL: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della colonna, raggiunto il valore massimo di colonne viene incrementato il valore della pagina e portato a zero il valore delle colonne
 *              - OLED_ADDRESSING_VERTICAL: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della pagina, raggiunto il valore massimo di pagine viene incrementato il valore della colonna e portato a zero il valore delle pagine
 *              - OLED_ADDRESSING_PAGE: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della colonna, raggiunto il valore massimo di colonne viene portato a zero il valore delle colonne ma il valore della pagina resta invariato
 * @return  void
 * @note    Solo per SSD1306
 */ 
#ifdef OLED_DRIVER_SSD1306
void GLCD_set_addressing_mode(unsigned char _mode);
#endif

/**
 * Imposta la modalit di scrolling del display
 * @param   _scrolling: modalit di scrolling (unsigned char):
 *              - OLED_HORIZONTAL_SCROLL_RIGHT
 *              - OLED_HORIZONTAL_SCROLL_LEFT
 *              - OLED_VERTICAL_HORIZONTAL_RIGHT
 *              - OLED_VERTICAL_HORIZONTAL_LEFT
 * @param   _page_start: pagina di inizio scrolling 0~7 (unsigned char)
 * @param   _page_end: pagina di fine scrolling 0~7 (unsigned char)
 * @param   _time: tempo di scrolling 1~7 (unsigned char)
 * @return  void
 * @note    Per lo scrolling verticale tutto il display verr spostato in alto ma solo le pagine selezionate verranno spostate anche a sinistra o destra.
 *          Per spostare solo l'intero display in alto selezionare una pagina vuota  \n
 *          Solo per SSD1306
 */ 
#ifdef OLED_DRIVER_SSD1306
void GLCD_scroll_set(unsigned char _scrolling, unsigned char _page_start, unsigned char _page_end, unsigned char _time);
#endif

/**
 * Attiva o disattiva lo scrolling con i parametri impostati tramite funzione GLCD_set_scroll()
 * @param   _on_off: OLED_SCROLL_ENABLED/OLED_SCROLL_DISABLED (unsigned char)
 * @return  void
 * @pre     Impostare il tipo di scrolling tramite funzione GLCD_set_scroll();
 * @post    Dopo aver disattivato lo scrolling il display va riscritto.
 * @note    Con scrolling attivo non  possibile scrivere nuovi valori sul display e cambiare i parametri di scroll      \n
 *          Solo per SSD1306
 */ 
#ifdef OLED_DRIVER_SSD1306
void GLCD_scroll_en(unsigned char _on_off);
#endif

/**
 * Cancella o riempie tutto il display
 * @param   _on_off: PIXEL_ON/PIXEL_OFF (unsigned char)
 * @return  void
 */ 
void GLCD_clear(unsigned char _on_off);

/**
 * Scrive un bit nell'indirizzo xy selezionato
 * @param   _x: indirizzo x: 1~X_RESOLUTION (unsigned char)
 * @param   _y: indirizzo y: 1~Y_RESOLUTION (unsigned char)
 * @param   _on_off: bit acceso o spento: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return  void
 * @note    In scrittura SPI non  possibile leggere il display, tutti i bit della pagina verrano scritti cancellando di fatto precedenti valori
 */
void GLCD_draw_pixel(unsigned char _x, unsigned char _y, unsigned char _on_off);

/**
 * Scrive un byte nella pagina selezionata all'indirizzo x selezionato
 * Utilizzato in funzioni di scrittura font e immagini
 * @param   _page: pagina da selezionare: 0~7 (unsigned char) 
 * @param   _address_x: indirizzo x: 0~127 (unsigned char)    
 * @param   _data: byte da scrivere (unsigned char)
 * @param   _preserve_old: PRESERVE/OVERWRITE (unsigned char) Preserva o sovrascrive i dati sulla stessa pagina
 * @param   _on_off: PIXEL_ON/PIXEL_OFF (unsigned char)
 * @return  void
 */
void GLCD_write_byte(unsigned char _page, unsigned char _x, unsigned char _data, unsigned char _preserve_old, unsigned char _on_off);

/**
 * Scrive un'immagine salvata in array sul display
 * @param   _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 * @return  void
 */
void GLCD_draw_image(const unsigned char  * _image);